/*
Target : ATTiny26L
Crystal: 4.0000Mhz

   ADC  Tiny26L       (ADC5,ADC6,ADC10).
   LPT(bit INIT)  .      
10-     8-   6      A,  2    B c   PB2.  
   379h      , ..   ,   
      - (    INT0).     . 
      ,           
   .

Design by Kulev Nikita. 2004.()
*/  

#include <tiny26.h>
#include <delay.h>

void port_init(void);
void init_devices(void);  
void adc_init(void);
void main(void);  
unsigned char counter; //     
unsigned char result;
unsigned char resa;
unsigned char resb;

///////////// programm /////////////////
void main(void)
{
 init_devices();
 while (1);
}

///////////// interrupt //////////////

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
{
 ADCSR|=0x40; // start conversion bit6 = 1 
 GIMSK = 0x00;// INT0       .
}


// ADC interrupt service routine
interrupt [ADC_INT] void adc_isr(void)
{
 ADCSR = 0x00; //disable adc
 
 result=ADCH; // 8-  
  
 result=~result;//     .
 resa=result & 0b00111111;//  6 
 PORTA|=resa;//   A 6   .
 resb=result>>6;//,       2 .
 PORTB|=resb;//  2    B
 DDRA=0b00111111; //     
 DDRB|=0b00000011;

 PORTB|=0X04; // ,   .  .
 delay_ms(1);//,     INT0,         .
 
 while (!(PINB.6==1));//   

 DDRA=0X00;//  A   
 DDRB=0x00;//  B
 PORTA = 0X00;//         . ..      . 
 PORTB = 0X00;//.    

 counter++; //   

 if (counter==1) 
  {
   ADMUX = 0x26;//ADC6 (   )
   goto exit;
  }

 if (counter==2) 
  {
   ADMUX = 0x2A;//ADC10
   goto exit;
  }

 if (counter==3) 
  {
   counter=0; 
   ADMUX = 0x25;//ADC5 (   )
   goto exit;
  }
 exit:
  ADCSR = 0x8D;//  
  GIMSK = 0x40;// IRQ 
  GIFR = 0xFF; //     
}                   


////////// initialization /////////////
void init_devices(void)
{ 
 counter = 0; // 
 #asm("CLI"); //disable all interrupts
 
 // Timer/Counter 0 initialization
 // Clock source: System Clock
 // Clock value: Timer 0 Stopped
 TCCR0=0x00;
 TCNT0=0x00;
 // Timer/Counter 1 initialization
 // Clock source: System Clock
 // Clock value: Timer 1 Stopped
 // Mode: Normal top=FFh
 // OC1A output: Disconnected
 // OC1B output: Disconnected
 PLLCSR=0x00;
 TCCR1A=0x00;
 TCCR1B=0x00;
 TCNT1=0x00;
 OCR1A=0x00;
 OCR1B=0x00;
 OCR1C=0x00;
 // Timer(s)/Counter(s) Interrupt(s) initialization
 TIMSK=0x00;

 // Universal Serial Interface initialization
 // Mode: Disabled
 // Clock source: Register & Counter=no clk.
 // USI Counter Overflow Interrupt: Off
 USICR=0x00;

 // Analog Comparator initialization
 // Analog Comparator: Off
 ACSR=0x80;

 // INT0 Mode: Rising Edge
 // Interrupt on any change on pins PA3, PA6, PA7 and PB4-7: Off
 // Interrupt on any change on pins PB0-3: On
 MCUCR=0b00000011;//(Rising INT0)
 GIFR=0b01000000;//  INT0 - bit6,  PinChange - bit5
  
 port_init();
 adc_init();
 GIMSK = 0x40; //  
 #asm("SEI"); //re-enable interrupts
}

void port_init(void)
{
 PORTA=0x00;
 DDRA=0x00; //    , ..    
 PORTB=0x00;
 DDRB=0x00; //    , ..    
}

void adc_init(void)
{ 
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: AVCC pin
// Only the 8 most significant bits of
// the AD conversion result are used
 ADCSR = 0x00; //disable adc
 ADMUX = 0b00100101; //00101 - select adc input ADC5. 
                     //1     - 8 BIT(ADLAR)  
                     //00    - REF = AVCC
                     //00100101(0x25) - ADC5; 00100110(0x26) - ADC6; 00101010(0x2A) - ADC10;
 ADCSR = 0x8D;//1000 1101
  /*
  Bit 7 - ADEN: Enabled ADC
  Bit 6 - ADSC: Start ADC (  Stop)
  Bit 5 - ADFR: Non FreeRun
  Bit 4 - ADIF: Interrupt Flag "1" - reset flag if "1" set 
  Bit 3 - ADIE: Enabled Interrupt
  Bits 2..0 - ADPS2..ADPS0: Prescaler 4000/32=125kHz
  */
  }